@@ -24,6 +24,7 @@
 #include <cairo/cairo.h>
 #include <guacamole/client.h>
 #include <guacamole/layer.h>
+#include <guacamole/mem.h>
 #include <guacamole/protocol.h>
 #include <guacamole/socket.h>
 #include <guacamole/timestamp.h>
@@ -103,6 +104,28 @@
  */
 #define GUAC_SURFACE_WEBP_BLOCK_SIZE 8
 
+void guac_common_surface_set_multitouch(guac_common_surface* surface,
+        int touches) {
+
+    pthread_mutex_lock(&surface->_lock);
+
+    surface->touches = touches;
+    guac_protocol_send_set_int(surface->socket, surface->layer,
+            GUAC_PROTOCOL_LAYER_PARAMETER_MULTI_TOUCH, touches);
+
+    pthread_mutex_unlock(&surface->_lock);
+
+}
+
+void guac_common_surface_set_lossless(guac_common_surface* surface,
+        int lossless) {
+
+    pthread_mutex_lock(&surface->_lock);
+    surface->lossless = lossless;
+    pthread_mutex_unlock(&surface->_lock);
+
+}
+
 void guac_common_surface_move(guac_common_surface* surface, int x, int y) {
 
     pthread_mutex_lock(&surface->_lock);
@@ -374,7 +397,7 @@ static unsigned int __guac_common_surface_calculate_framerate(
     int x, y;
 
     /* Calculate heat map dimensions */
-    int heat_width = GUAC_COMMON_SURFACE_HEAT_DIMENSION(surface->width);
+    size_t heat_width = GUAC_COMMON_SURFACE_HEAT_DIMENSION(surface->width);
 
     /* Calculate minimum X/Y coordinates intersecting given rect */
     int min_x = rect->x / GUAC_COMMON_SURFACE_HEAT_CELL_SIZE;
@@ -521,6 +544,10 @@ static int __guac_common_surface_png_optimality(guac_common_surface* surface,
 static int __guac_common_surface_should_use_jpeg(guac_common_surface* surface,
         const guac_common_rect* rect) {
 
+    /* Do not use JPEG if lossless quality is required */
+    if (surface->lossless)
+        return 0;
+
     /* Calculate the average framerate for the given rect */
     int framerate = __guac_common_surface_calculate_framerate(surface, rect);
 
@@ -589,7 +616,7 @@ static void __guac_common_surface_touch_rect(guac_common_surface* surface,
     int x, y;
 
     /* Calculate heat map dimensions */
-    int heat_width = GUAC_COMMON_SURFACE_HEAT_DIMENSION(surface->width);
+    size_t heat_width = GUAC_COMMON_SURFACE_HEAT_DIMENSION(surface->width);
 
     /* Calculate minimum X/Y coordinates intersecting given rect */
     int min_x = rect->x / GUAC_COMMON_SURFACE_HEAT_CELL_SIZE;
@@ -1201,11 +1228,11 @@ guac_common_surface* guac_common_surface_alloc(guac_client* client,
         guac_socket* socket, const guac_layer* layer, int w, int h) {
 
     /* Calculate heat map dimensions */
-    int heat_width = GUAC_COMMON_SURFACE_HEAT_DIMENSION(w);
-    int heat_height = GUAC_COMMON_SURFACE_HEAT_DIMENSION(h);
+    size_t heat_width = GUAC_COMMON_SURFACE_HEAT_DIMENSION(w);
+    size_t heat_height = GUAC_COMMON_SURFACE_HEAT_DIMENSION(h);
 
     /* Init surface */
-    guac_common_surface* surface = calloc(1, sizeof(guac_common_surface));
+    guac_common_surface* surface = guac_mem_zalloc(sizeof(guac_common_surface));
     surface->client = client;
     surface->socket = socket;
     surface->layer = layer;
@@ -1218,10 +1245,10 @@ guac_common_surface* guac_common_surface_alloc(guac_client* client,
 
     /* Create corresponding Cairo surface */
     surface->stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, w);
-    surface->buffer = calloc(h, surface->stride);
+    surface->buffer = guac_mem_zalloc(h, surface->stride);
 
     /* Create corresponding heat map */
-    surface->heat_map = calloc(heat_width * heat_height,
+    surface->heat_map = guac_mem_zalloc(heat_width, heat_height,
             sizeof(guac_common_surface_heat_cell));
 
     /* Reset clipping rect */
@@ -1248,9 +1275,9 @@ void guac_common_surface_free(guac_common_surface* surface) {
 
     pthread_mutex_destroy(&surface->_lock);
 
-    free(surface->heat_map);
-    free(surface->buffer);
-    free(surface);
+    guac_mem_free(surface->heat_map);
+    guac_mem_free(surface->buffer);
+    guac_mem_free(surface);
 
 }
 
@@ -1273,8 +1300,8 @@ void guac_common_surface_resize(guac_common_surface* surface, int w, int h) {
     int sy = 0;
 
     /* Calculate heat map dimensions */
-    int heat_width = GUAC_COMMON_SURFACE_HEAT_DIMENSION(w);
-    int heat_height = GUAC_COMMON_SURFACE_HEAT_DIMENSION(h);
+    size_t heat_width = GUAC_COMMON_SURFACE_HEAT_DIMENSION(w);
+    size_t heat_height = GUAC_COMMON_SURFACE_HEAT_DIMENSION(h);
 
     /* Copy old surface data */
     old_buffer = surface->buffer;
@@ -1285,7 +1312,7 @@ void guac_common_surface_resize(guac_common_surface* surface, int w, int h) {
     surface->width  = w;
     surface->height = h;
     surface->stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, w);
-    surface->buffer = calloc(h, surface->stride);
+    surface->buffer = guac_mem_zalloc(h, surface->stride);
     __guac_common_bound_rect(surface, &surface->clip_rect, NULL, NULL);
 
     /* Copy relevant old data */
@@ -1293,11 +1320,11 @@ void guac_common_surface_resize(guac_common_surface* surface, int w, int h) {
     __guac_common_surface_put(old_buffer, old_stride, &sx, &sy, surface, &old_rect, 1);
 
     /* Free old data */
-    free(old_buffer);
+    guac_mem_free(old_buffer);
 
     /* Allocate completely new heat map (can safely discard old stats) */
-    free(surface->heat_map);
-    surface->heat_map = calloc(heat_width * heat_height,
+    guac_mem_free(surface->heat_map);
+    surface->heat_map = guac_mem_alloc(heat_width, heat_height,
             sizeof(guac_common_surface_heat_cell));
 
     /* Resize dirty rect to fit new surface dimensions */
@@ -1793,7 +1820,8 @@ static void __guac_common_surface_flush_to_webp(guac_common_surface* surface,
         /* Send WebP for rect */
         guac_client_stream_webp(surface->client, socket, GUAC_COMP_OVER, layer,
                 surface->dirty_rect.x, surface->dirty_rect.y, rect,
-                guac_common_surface_suggest_quality(surface->client), 0);
+                guac_common_surface_suggest_quality(surface->client),
+                surface->lossless ? 1 : 0);
 
         cairo_surface_destroy(rect);
         surface->realized = 1;
@@ -1962,8 +1990,8 @@ void guac_common_surface_flush(guac_common_surface* surface) {
 
 }
 
-void guac_common_surface_dup(guac_common_surface* surface, guac_user* user,
-        guac_socket* socket) {
+void guac_common_surface_dup(guac_common_surface* surface,
+        guac_client* client, guac_socket* socket) {
 
     pthread_mutex_lock(&surface->_lock);
 
@@ -1981,6 +2009,11 @@ void guac_common_surface_dup(guac_common_surface* surface, guac_user* user,
         guac_protocol_send_move(socket, surface->layer,
                 surface->parent, surface->x, surface->y, surface->z);
 
+        /* Synchronize multi-touch support level */
+        guac_protocol_send_set_int(surface->socket, surface->layer,
+                GUAC_PROTOCOL_LAYER_PARAMETER_MULTI_TOUCH,
+                surface->touches);
+
     }
 
     /* Sync size to new socket */
@@ -1996,7 +2029,7 @@ void guac_common_surface_dup(guac_common_surface* surface, guac_user* user,
                 surface->width, surface->height, surface->stride);
 
         /* Send PNG for rect */
-        guac_user_stream_png(user, socket, GUAC_COMP_OVER, surface->layer,
+        guac_client_stream_png(client, socket, GUAC_COMP_OVER, surface->layer,
                 0, 0, rect);
         cairo_surface_destroy(rect);
 
@@ -2006,4 +2039,3 @@ complete:
     pthread_mutex_unlock(&surface->_lock);
 
 }
-
