@@ -26,6 +26,7 @@
 
 #include <guacamole/client.h>
 #include <guacamole/error.h>
+#include <guacamole/mem.h>
 #include <guacamole/parser.h>
 #include <guacamole/plugin.h>
 #include <guacamole/protocol.h>
@@ -106,7 +107,7 @@ static void* guacd_user_thread(void* data) {
     /* Clean up */
     guac_socket_free(socket);
     guac_user_free(user);
-    free(params);
+    guac_mem_free(params);
 
     return NULL;
 
@@ -130,7 +131,7 @@ static void* guacd_user_thread(void* data) {
  */
 static void guacd_proc_add_user(guacd_proc* proc, int fd, int owner) {
 
-    guacd_user_thread_params* params = malloc(sizeof(guacd_user_thread_params));
+    guacd_user_thread_params* params = guac_mem_alloc(sizeof(guacd_user_thread_params));
     params->proc = proc;
     params->fd = fd;
     params->owner = owner;
@@ -287,6 +288,26 @@ static int guacd_timed_client_free(guac_client* client, int timeout) {
     return !free_operation.completed;
 }
 
+/**
+ * A reference to the current guacd process.
+ */
+guacd_proc* guacd_proc_self = NULL;
+
+/**
+ * A signal handler that will be invoked when a signal is caught telling this
+ * guacd process to immediately exit.
+ *
+ * @param signal
+ *     The signal that was received. Unused in this function since only
+ *     signals that should result in stopping the proc should invoke this.
+ */
+static void signal_stop_handler(int signal) {
+
+    /* Stop the current guacd proc */
+    guacd_proc_stop(guacd_proc_self);
+
+}
+
 /**
  * Starts protocol-specific handling on the given process by loading the client
  * plugin for that protocol. This function does NOT return. It initializes the
@@ -333,6 +354,13 @@ static void guacd_exec_proc(guacd_proc* proc, const char* protocol) {
     /* Enable keep alive on the broadcast socket */
     guac_socket_require_keep_alive(client->socket);
 
+    guacd_proc_self = proc;
+
+    /* Clean up and exit if SIGINT or SIGTERM signals are caught */
+    struct sigaction signal_stop_action = { .sa_handler = signal_stop_handler };
+    sigaction(SIGINT, &signal_stop_action, NULL);
+    sigaction(SIGTERM, &signal_stop_action, NULL);
+
     /* Add each received file descriptor as a new user */
     int received_fd;
     while ((received_fd = guacd_recv_fd(proc->fd_socket)) != -1) {
@@ -382,7 +410,7 @@ cleanup_process:
 
     /* Free up all internal resources outside the client */
     close(proc->fd_socket);
-    free(proc);
+    guac_mem_free(proc);
 
     exit(result);
 
@@ -402,7 +430,7 @@ guacd_proc* guacd_create_proc(const char* protocol) {
     int child_socket = sockets[1];
 
     /* Allocate process */
-    guacd_proc* proc = calloc(1, sizeof(guacd_proc));
+    guacd_proc* proc = guac_mem_zalloc(sizeof(guacd_proc));
     if (proc == NULL) {
         close(parent_socket);
         close(child_socket);
@@ -415,7 +443,7 @@ guacd_proc* guacd_create_proc(const char* protocol) {
         guacd_log_guac_error(GUAC_LOG_ERROR, "Unable to create client");
         close(parent_socket);
         close(child_socket);
-        free(proc);
+        guac_mem_free(proc);
         return NULL;
     }
 
@@ -429,7 +457,7 @@ guacd_proc* guacd_create_proc(const char* protocol) {
         close(parent_socket);
         close(child_socket);
         guac_client_free(proc->client);
-        free(proc);
+        guac_mem_free(proc);
         return NULL;
     }
 
@@ -458,8 +486,43 @@ guacd_proc* guacd_create_proc(const char* protocol) {
 
 }
 
+/**
+ * Kill the provided child guacd process. This function must be called by the
+ * parent process, and will block until all processes associated with the
+ * child process have terminated.
+ *
+ * @param proc
+ *     The child guacd process to kill.
+ */
+static void guacd_proc_kill(guacd_proc* proc) {
+
+    /* Request orderly termination of process */
+    if (kill(proc->pid, SIGTERM))
+        guacd_log(GUAC_LOG_DEBUG, "Unable to request termination of "
+                "client process: %s ", strerror(errno));
+
+    /* Wait for all processes within process group to terminate */
+    pid_t child_pid;
+    while ((child_pid = waitpid(-proc->pid, NULL, 0)) > 0 || errno == EINTR) {
+        guacd_log(GUAC_LOG_DEBUG, "Child process %i of connection \"%s\" has terminated",
+            child_pid, proc->client->connection_id);
+    }
+
+    guacd_log(GUAC_LOG_DEBUG, "All child processes for connection \"%s\" have been terminated.",
+        proc->client->connection_id);
+
+}
+
 void guacd_proc_stop(guacd_proc* proc) {
 
+    /* A non-zero PID means that this is the parent process */
+    if (proc->pid != 0) {
+        guacd_proc_kill(proc);
+        return;
+    }
+
+    /* Otherwise, this is the child process */
+
     /* Signal client to stop */
     guac_client_stop(proc->client);
 
@@ -473,4 +536,3 @@ void guacd_proc_stop(guacd_proc* proc) {
     close(proc->fd_socket);
 
 }
-
