@@ -32,6 +32,7 @@
 #include "layer-types.h"
 #include "object-types.h"
 #include "pool-types.h"
+#include "rwlock.h"
 #include "socket-types.h"
 #include "stream-types.h"
 #include "timestamp-types.h"
@@ -42,20 +43,29 @@
 
 #include <pthread.h>
 #include <stdarg.h>
+#include <time.h>
 
 struct guac_client {
 
     /**
-     * The guac_socket structure to be used to communicate with all connected
-     * web-clients (users). Unlike the user-level guac_socket, this guac_socket
-     * will broadcast instructions to all connected users simultaneously.  It
-     * is expected that the implementor of any Guacamole proxy client will
-     * provide their own mechanism of I/O for their protocol. The guac_socket
-     * structure is used only to communicate conveniently with the Guacamole
-     * web-client.
+     * The guac_socket structure to be used to communicate with all non-pending
+     * connected web-clients (users). Unlike the user-level guac_socket, this
+     * guac_socket will broadcast instructions to all non-pending connected users
+     * simultaneously. It is expected that the implementor of any Guacamole proxy
+     * client will provide their own mechanism of I/O for their protocol. The
+     * guac_socket structure is used only to communicate conveniently with the
+     * Guacamole web-client.
      */
     guac_socket* socket;
 
+    /**
+     * The guac_socket structure to be used to communicate with all pending
+     * connected web-clients (users). Aside from operating on a different
+     * subset of users, this socket has all the same behavior and semantics as
+     * the non-pending socket.
+     */
+    guac_socket* pending_socket;
+
     /**
      * The current state of the client. When the client is first allocated,
      * this will be initialized to GUAC_CLIENT_RUNNING. It will remain at
@@ -162,7 +172,7 @@ struct guac_client {
      * Lock which is acquired when the users list is being manipulated, or when
      * the users list is being iterated.
      */
-    pthread_rwlock_t __users_lock;
+    guac_rwlock __users_lock;
 
     /**
      * The first user within the list of all connected users, or NULL if no
@@ -170,6 +180,37 @@ struct guac_client {
      */
     guac_user* __users;
 
+    /**
+     * Lock which is acquired when the pending users list is being manipulated,
+     * or when the pending users list is being iterated.
+     */
+    guac_rwlock __pending_users_lock;
+
+    /**
+     * A timer that will periodically synchronize the list of pending users,
+     * emptying the list once synchronization is complete. Only for internal
+     * use within the client. This will be NULL until the first user joins
+     * the connection, as it is lazily instantiated at that time.
+     */
+    timer_t __pending_users_timer;
+
+    /**
+     * A flag storing the current state of the pending users timer.
+     */
+    int __pending_users_timer_state;
+
+    /**
+     * A mutex that must be acquired before modifying or checking the value of
+     * the timer state.
+     */
+    pthread_mutex_t __pending_users_timer_mutex;
+
+    /**
+     * The first user within the list of connected users who have not yet had
+     * their connection states synchronized after joining.
+     */
+    guac_user* __pending_users;
+
     /**
      * The user that first created this connection. This user will also have
      * their "owner" flag set to a non-zero value. If the owner has left the
@@ -206,6 +247,22 @@ struct guac_client {
      */
     guac_user_join_handler* join_handler;
 
+    /**
+     * A handler that will be run prior to pending users being promoted to full
+     * users. Any required pending user operations should be performed using
+     * the client's pending user socket.
+     *
+     * Example:
+     * @code
+     *     int join_pending_handler(guac_client* client);
+     *
+     *     int guac_client_init(guac_client* client) {
+     *         client->join_pending_handler = join_pending_handler;
+     *     }
+     * @endcode
+     */
+    guac_client_join_pending_handler* join_pending_handler;
+
     /**
      * Handler for leave events, called whenever a new user is leaving an
      * active connection.
@@ -446,6 +503,26 @@ void guac_client_remove_user(guac_client* client, guac_user* user);
 void guac_client_foreach_user(guac_client* client,
         guac_user_callback* callback, void* data);
 
+/**
+ * Calls the given function on all pending users of the given client. The
+ * function will be given a reference to a guac_user and the specified
+ * arbitrary data. The value returned by the callback will be ignored.
+ *
+ * This function is reentrant, but the pending user list MUST NOT be manipulated
+ * within the same thread as a callback to this function.
+ *
+ * @param client
+ *     The client whose users should be iterated.
+ *
+ * @param callback
+ *     The function to call for each pending user.
+ *
+ * @param data
+ *     Arbitrary data to pass to the callback each time it is invoked.
+ */
+void guac_client_foreach_pending_user(guac_client* client,
+        guac_user_callback* callback, void* data);
+
 /**
  * Calls the given function with the currently-connected user that is marked as
  * the owner. The owner of a connection is the user that established the
@@ -708,6 +785,21 @@ void guac_client_stream_webp(guac_client* client, guac_socket* socket,
         guac_composite_mode mode, const guac_layer* layer, int x, int y,
         cairo_surface_t* surface, int quality, int lossless);
 
+/**
+ * Returns whether the owner of the given client supports the "msg"
+ * instruction, returning non-zero if the client owner does support the
+ * instruction, or zero if the owner does not.
+ * 
+ * @param client
+ *     The Guacamole client whose owner should be checked for supporting
+ *     the "msg" instruction.
+ * 
+ * @return 
+ *     Non-zero if the owner of the given client supports the "msg"
+ *     instruction, zero otherwise.
+ */
+int guac_client_owner_supports_msg(guac_client* client);
+
 /**
  * Returns whether the owner of the given client supports the "required"
  * instruction, returning non-zero if the client owner does support the
@@ -723,6 +815,42 @@ void guac_client_stream_webp(guac_client* client, guac_socket* socket,
  */
 int guac_client_owner_supports_required(guac_client* client);
 
+/**
+ * Notifies the owner of the given client that a user has joined the connection,
+ * and returns zero if the message was sent successfully, or non-zero if the
+ * notification failed.
+ *
+ * @param client
+ *     The Guacamole Client whose owner should be notified of a user joining
+ *     the connection.
+ *
+ * @param joiner
+ *     The Guacamole User who joined the connection.
+ * 
+ * @return
+ *     Zero if the notification to the owner was sent successfully, or non-zero
+ *     if an error occurred.
+ */
+int guac_client_owner_notify_join(guac_client* client, guac_user* joiner);
+
+/**
+ * Notifies the owner of the given client that a user has left the connection,
+ * and returns zero if the message was sent successfully, or non-zero if the
+ * notification failed.
+ *
+ * @param client
+ *     The Guacamole Client whose owner should be notified of a user leaving
+ *     the connection.
+ *
+ * @param quitter
+ *     The Guacamole User who left the connection.
+ * 
+ * @return
+ *     Zero if the notification to the owner was sent successfully, or non-zero
+ *     if an error occurred.
+ */
+int guac_client_owner_notify_leave(guac_client* client, guac_user* quitter);
+
 /**
  * Returns whether all users of the given client support WebP. If any user does
  * not support WebP, or the server cannot encode WebP images, zero is returned.
