@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #ifndef _GUAC_SOCKET_H
@@ -29,6 +26,7 @@
  * @file socket.h
  */
 
+#include "client-types.h"
 #include "socket-constants.h"
 #include "socket-fntypes.h"
 #include "socket-types.h"
@@ -52,13 +50,28 @@ struct guac_socket {
 
     /**
      * Handler which will be called whenever data is written to this socket.
-     * Note that because guac_socket automatically buffers written data, this
-     * handler might only get called when the socket is flushed.
      */
     guac_socket_write_handler* write_handler;
 
     /**
-     * Handler which will be called whenever guac_socket_select is invoked
+     * Handler which will be called whenever this socket needs to be flushed.
+     */
+    guac_socket_flush_handler* flush_handler;
+
+    /**
+     * Handler which will be called whenever a socket needs to be acquired for
+     * exclusive access, such as when an instruction is about to be written.
+     */
+    guac_socket_lock_handler* lock_handler;
+
+    /**
+     * Handler which will be called whenever exclusive access to a socket is
+     * being released, such as when an instruction has finished being written.
+     */
+    guac_socket_unlock_handler* unlock_handler;
+
+    /**
+     * Handler which will be called whenever guac_socket_select() is invoked
      * on this socket.
      */
     guac_socket_select_handler* select_handler;
@@ -90,52 +103,6 @@ struct guac_socket {
      */
     int __ready_buf[3];
 
-    /**
-     * The number of bytes currently in the main write buffer.
-     */
-    int __written;
-
-    /**
-     * The main write buffer. Bytes written go here before being flushed
-     * to the open file descriptor.
-     */
-    char __out_buf[GUAC_SOCKET_OUTPUT_BUFFER_SIZE];
-
-    /**
-     * Pointer to the first character of the current in-progress instruction
-     * within the buffer.
-     */
-    char* __instructionbuf_unparsed_start;
-
-    /**
-     * Pointer to the first unused section of the instruction buffer.
-     */
-    char* __instructionbuf_unparsed_end;
-
-    /**
-     * The instruction buffer. This is essentially the input buffer,
-     * provided as a convenience to be used to buffer instructions until
-     * those instructions are complete and ready to be parsed.
-     */
-    char __instructionbuf[32768];
-
-    /**
-     * Whether instructions should be guaranteed atomic across threads using
-     * locks. By default, thread safety is disabled on sockets.
-     */
-    int __threadsafe_instructions;
-
-    /**
-     * Lock which is acquired when an instruction is being written, and
-     * released when the instruction is finished being written.
-     */
-    pthread_mutex_t __instruction_write_lock;
-
-    /**
-     * Lock which is acquired when the buffer is being modified or flushed.
-     */
-    pthread_mutex_t __buffer_lock;
-
     /**
      * Whether automatic keep-alive is enabled.
      */
@@ -164,66 +131,36 @@ guac_socket* guac_socket_alloc();
  */
 void guac_socket_free(guac_socket* socket);
 
-/**
- * Declares that the given socket must behave in a threadsafe way. Calling
- * this function on a socket guarantees that the socket will send instructions
- * atomically. Without automatic threadsafe sockets, multiple threads writing
- * to the same socket must ensure that instructions will not potentially
- * overlap.
- *
- * @param socket The guac_socket to declare as threadsafe.
- */
-void guac_socket_require_threadsafe(guac_socket* socket);
-
 /**
  * Declares that the given socket must automatically send a keep-alive ping
  * to ensure neither side of the socket times out while the socket is open.
- * This ping will take the form of a "nop" instruction. Enabling keep-alive
- * automatically enables threadsafety.
+ * This ping will take the form of a "nop" instruction.
  *
- * @param socket The guac_socket to declare as requiring an automatic
- *               keep-alive ping.
+ * @param socket
+ *     The guac_socket to declare as requiring an automatic keep-alive ping.
  */
 void guac_socket_require_keep_alive(guac_socket* socket);
 
 /**
- * Marks the beginning of a Guacamole protocol instruction. If threadsafety
- * is enabled on the socket, other instructions will be blocked from sending
- * until this instruction is complete.
+ * Marks the beginning of a Guacamole protocol instruction.
  *
- * @param socket The guac_socket beginning an instruction.
+ * @param socket
+ *     The guac_socket beginning an instruction.
  */
 void guac_socket_instruction_begin(guac_socket* socket);
 
 /**
- * Marks the end of a Guacamole protocol instruction. If threadsafety
- * is enabled on the socket, other instructions will be allowed to send.
+ * Marks the end of a Guacamole protocol instruction.
  *
- * @param socket The guac_socket ending an instruction.
+ * @param socket
+ *     The guac_socket ending an instruction.
  */
 void guac_socket_instruction_end(guac_socket* socket);
 
-/**
- * Marks the beginning of a socket's buffer modification. If threadsafety is
- * enabled on the socket, other functions which modify the buffer will be
- * blocked until this modification is complete.
- *
- * @param socket The guac_socket whose buffer is being updated.
- */
-void guac_socket_update_buffer_begin(guac_socket* socket);
-
-/**
- * Marks the end of a socket's buffer modification. If threadsafety is enabled
- * on the socket, other functions which modify the buffer will now be allowed
- * to continue.
- *
- * @param socket The guac_socket whose buffer is done being updated.
- */
-void guac_socket_update_buffer_end(guac_socket* socket);
-
 /**
  * Allocates and initializes a new guac_socket object with the given open
- * file descriptor.
+ * file descriptor. The file descriptor will be automatically closed when
+ * the allocated guac_socket is freed.
  *
  * If an error occurs while allocating the guac_socket object, NULL is returned,
  * and guac_error is set appropriately.
@@ -237,11 +174,19 @@ guac_socket* guac_socket_open(int fd);
 
 /**
  * Allocates and initializes a new guac_socket which writes all data via
- * nest instructions to the given existing, open guac_socket.
+ * nest instructions to the given existing, open guac_socket. Freeing the
+ * returned guac_socket has no effect on the underlying, nested guac_socket.
  *
  * If an error occurs while allocating the guac_socket object, NULL is returned,
  * and guac_error is set appropriately.
  *
+ * @deprecated
+ *     The "nest" instruction and the corresponding guac_socket
+ *     implementation are no longer necessary, having been replaced by
+ *     the streaming instructions ("blob", "ack", "end"). Code using nested
+ *     sockets or the "nest" instruction should instead write to a normal
+ *     socket directly.
+ *
  * @param parent The guac_socket this new guac_socket should write nest
  *               instructions to.
  * @param index The stream index to use for the written nest instructions.
@@ -251,6 +196,90 @@ guac_socket* guac_socket_open(int fd);
  */
 guac_socket* guac_socket_nest(guac_socket* parent, int index);
 
+/**
+ * Allocates and initializes a new guac_socket which delegates all socket
+ * operations to the given primary socket, while simultaneously duplicating all
+ * written data to the secondary socket. Freeing the returned guac_socket will
+ * free both primary and secondary sockets.
+ *
+ * Return values (error codes) will come only from the primary socket. Locks
+ * (like those used by guac_socket_instruction_begin() and
+ * guac_socket_instruction_end()) will affect only the primary socket.
+ *
+ * If an error occurs while allocating the guac_socket object, NULL is returned,
+ * and guac_error is set appropriately.
+ *
+ * @param primary
+ *     The primary guac_socket to which all socket operations should be
+ *     delegated. The error codes returned by socket operations, if any, will
+ *     always come from this socket. This socket will also be the only socket
+ *     locked when instructions begin (or unlocked when instructions end).
+ *
+ * @param secondary
+ *     The secondary guac_socket to which all data written to the primary
+ *     guac_socket should be copied. If an error prevents the write from
+ *     succeeding, that error will be ignored. Only errors from the primary
+ *     guac_socket will be acknowledged.
+ *
+ * @return
+ *     A newly allocated guac_socket object associated with the given primary
+ *     and secondary sockets, or NULL if an error occurs while allocating the
+ *     guac_socket object.
+ */
+guac_socket* guac_socket_tee(guac_socket* primary, guac_socket* secondary);
+
+/**
+ * Allocates and initializes a new guac_socket which duplicates all
+ * instructions written across the sockets of each connected user of the
+ * given guac_client. The returned socket is a write-only socket. Attempts
+ * to read from the socket will fail. If a write occurs while no users are
+ * connected, that write will simply be dropped.
+ *
+ * Return values (error codes) from each user's socket will not affect the
+ * in-progress write, but each failing user will be forcibly stopped with
+ * guac_user_stop().
+ *
+ * If an error occurs while allocating the guac_socket object, NULL is returned,
+ * and guac_error is set appropriately.
+ *
+ * @param client
+ *     The client associated with the group of connected users across which
+ *     duplicates of all instructions should be written.
+ *
+ * @return
+ *     A write-only guac_socket object which broadcasts copies of all
+ *     instructions written across all non-pending connected users of the given
+ *     guac_client, or NULL if an error occurs while allocating the guac_socket
+ *     object.
+ */
+guac_socket* guac_socket_broadcast(guac_client* client);
+
+/**
+ * Allocates and initializes a new guac_socket which duplicates all
+ * instructions written across the sockets of each pending connected
+ * user of the given guac_client. The returned socket is a write-only socket.
+ * Attempts to read from the socket will fail.  If a write occurs while no
+ * users are connected, that write will simply be dropped.
+ *
+ * Return values (error codes) from each user's socket will not affect the
+ * in-progress write, but each failing user will be forcibly stopped with
+ * guac_user_stop().
+ *
+ * If an error occurs while allocating the guac_socket object, NULL is returned,
+ * and guac_error is set appropriately.
+ *
+ * @param client
+ *     The client associated with the group of pending users across which
+ *     duplicates of all instructions should be written.
+ *
+ * @return
+ *     A write-only guac_socket object which broadcasts copies of all
+ *     instructions written across all pending connected users of the given
+ *     guac_client, or NULL if an error occurs while allocating the guac_socket
+ *     object.
+ */
+guac_socket* guac_socket_broadcast_pending(guac_client* client);
+
 /**
  * Writes the given unsigned int to the given guac_socket object. The data
  * written may be buffered until the buffer is flushed automatically or
@@ -268,9 +297,7 @@ ssize_t guac_socket_write_int(guac_socket* socket, int64_t i);
 /**
  * Writes the given string to the given guac_socket object. The data
  * written may be buffered until the buffer is flushed automatically or
- * manually. Note that if the string can contain characters used
- * internally by the Guacamole protocol (commas, semicolons, or
- * backslashes) it will need to be escaped.
+ * manually.
  *
  * If an error occurs while writing, a non-zero value is returned, and
  * guac_error is set appropriately.
@@ -284,9 +311,9 @@ ssize_t guac_socket_write_string(guac_socket* socket, const char* str);
 /**
  * Writes the given binary data to the given guac_socket object as base64-
  * encoded data. The data written may be buffered until the buffer is flushed
- * automatically or manually. Beware that because base64 data is buffered
+ * automatically or manually. Beware that, because base64 data is buffered
  * on top of the write buffer already used, a call to guac_socket_flush_base64()
- * must be made before non-base64 writes (or writes of an independent block of
+ * MUST be made before non-base64 writes (or writes of an independent block of
  * base64 data) can be made.
  *
  * If an error occurs while writing, a non-zero value is returned, and
@@ -300,8 +327,8 @@ ssize_t guac_socket_write_string(guac_socket* socket, const char* str);
 ssize_t guac_socket_write_base64(guac_socket* socket, const void* buf, size_t count);
 
 /**
- * Writes the given data to the specified socket. The data written is not
- * buffered, and will be sent immediately.
+ * Writes the given data to the specified socket. The data written may be
+ * buffered until the buffer is flushed automatically or manually.
  *
  * If an error occurs while writing, a non-zero value is returned, and
  * guac_error is set appropriately.
