@@ -19,6 +19,7 @@
 
 #include "config.h"
 
+#include "guacamole/mem.h"
 #include "guacamole/client.h"
 #include "guacamole/error.h"
 #include "guacamole/parser.h"
@@ -372,8 +373,8 @@ int guac_user_handle_connection(guac_user* user, int usec_timeout) {
     guac_free_mimetypes((char **) user->info.video_mimetypes);
     
     /* Free name and timezone info. */
-    free((char *) user->info.name);
-    free((char *) user->info.timezone);
+    guac_mem_free_const(user->info.name);
+    guac_mem_free_const(user->info.timezone);
     
     guac_parser_free(parser);
 
