@@ -22,12 +22,14 @@
 #include "encode-jpeg.h"
 #include "encode-png.h"
 #include "encode-webp.h"
+#include "guacamole/mem.h"
 #include "guacamole/client.h"
 #include "guacamole/object.h"
 #include "guacamole/pool.h"
 #include "guacamole/protocol.h"
 #include "guacamole/socket.h"
 #include "guacamole/stream.h"
+#include "guacamole/string.h"
 #include "guacamole/timestamp.h"
 #include "guacamole/user.h"
 #include "id.h"
@@ -40,13 +42,13 @@
 
 guac_user* guac_user_alloc() {
 
-    guac_user* user = calloc(1, sizeof(guac_user));
+    guac_user* user = guac_mem_zalloc(sizeof(guac_user));
     int i;
 
     /* Generate ID */
     user->user_id = guac_generate_id(GUAC_USER_ID_PREFIX);
     if (user->user_id == NULL) {
-        free(user);
+        guac_mem_free(user);
         return NULL;
     }
 
@@ -59,8 +61,8 @@ guac_user* guac_user_alloc() {
     user->__stream_pool = guac_pool_alloc(0);
 
     /* Initialze streams */
-    user->__input_streams = malloc(sizeof(guac_stream) * GUAC_USER_MAX_STREAMS);
-    user->__output_streams = malloc(sizeof(guac_stream) * GUAC_USER_MAX_STREAMS);
+    user->__input_streams = guac_mem_alloc(sizeof(guac_stream), GUAC_USER_MAX_STREAMS);
+    user->__output_streams = guac_mem_alloc(sizeof(guac_stream), GUAC_USER_MAX_STREAMS);
 
     for (i=0; i<GUAC_USER_MAX_STREAMS; i++) {
         user->__input_streams[i].index = GUAC_USER_CLOSED_STREAM_INDEX;
@@ -71,7 +73,7 @@ guac_user* guac_user_alloc() {
     user->__object_pool = guac_pool_alloc(0);
 
     /* Initialize objects */
-    user->__objects = malloc(sizeof(guac_object) * GUAC_USER_MAX_OBJECTS);
+    user->__objects = guac_mem_alloc(sizeof(guac_object), GUAC_USER_MAX_OBJECTS);
     for (i=0; i<GUAC_USER_MAX_OBJECTS; i++)
         user->__objects[i].index = GUAC_USER_UNDEFINED_OBJECT_INDEX;
 
@@ -82,21 +84,21 @@ guac_user* guac_user_alloc() {
 void guac_user_free(guac_user* user) {
 
     /* Free streams */
-    free(user->__input_streams);
-    free(user->__output_streams);
+    guac_mem_free(user->__input_streams);
+    guac_mem_free(user->__output_streams);
 
     /* Free stream pool */
     guac_pool_free(user->__stream_pool);
 
     /* Free objects */
-    free(user->__objects);
+    guac_mem_free(user->__objects);
 
     /* Free object pool */
     guac_pool_free(user->__object_pool);
 
     /* Clean up user */
-    free(user->user_id);
-    free(user);
+    guac_mem_free(user->user_id);
+    guac_mem_free(user);
 
 }
 
@@ -377,12 +379,12 @@ char* guac_user_parse_args_string(guac_user* user, const char** arg_names,
         guac_user_log(user, GUAC_LOG_DEBUG, "Parameter \"%s\" omitted. Using "
                 "default value of \"%s\".", arg_names[index], default_value);
 
-        return strdup(default_value);
+        return guac_strdup(default_value);
 
     }
 
     /* Otherwise use provided value */
-    return strdup(value);
+    return guac_strdup(value);
 
 }
 
