@@ -21,6 +21,7 @@
 #include "channels/audio-input/audio-buffer.h"
 #include "channels/cliprdr.h"
 #include "channels/disp.h"
+#include "channels/pipe-svc.h"
 #include "config.h"
 #include "fs.h"
 #include "log.h"
@@ -36,6 +37,7 @@
 
 #include <guacamole/audio.h>
 #include <guacamole/client.h>
+#include <guacamole/mem.h>
 #include <guacamole/recording.h>
 
 #include <dirent.h>
@@ -78,6 +80,61 @@ static int is_writable_directory(const char* path) {
 
 }
 
+/**
+ * Add the provided user to the provided audio stream.
+ *
+ * @param user
+ *    The pending user who should be added to the audio stream.
+ *
+ * @param data
+ *    The audio stream that the user should be added to.
+ *
+ * @return
+ *     Always NULL.
+ */
+static void* guac_rdp_sync_pending_user_audio(guac_user* user, void* data) {
+
+    /* Add the user to the stream */
+    guac_audio_stream* audio = (guac_audio_stream*) data;
+    guac_audio_stream_add_user(audio, user);
+
+    return NULL;
+
+}
+
+/**
+ * A pending join handler implementation that will synchronize the connection
+ * state for all pending users prior to them being promoted to full user.
+ *
+ * @param client
+ *     The client whose pending users are about to be promoted.
+ *
+ * @return
+ *     Always zero.
+ */
+static int guac_rdp_join_pending_handler(guac_client* client) {
+
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+    guac_socket* broadcast_socket = client->pending_socket;
+
+    /* Synchronize any audio stream for each pending user */
+    if (rdp_client->audio)
+        guac_client_foreach_pending_user(
+            client, guac_rdp_sync_pending_user_audio, rdp_client->audio);
+
+    /* Bring user up to date with any registered static channels */
+    guac_rdp_pipe_svc_send_pipes(client, broadcast_socket);
+
+    /* Synchronize with current display */
+    if (rdp_client->display != NULL) {
+        guac_common_display_dup(rdp_client->display, client, broadcast_socket);
+        guac_socket_flush(broadcast_socket);
+    }
+
+    return 0;
+
+}
+
 int guac_client_init(guac_client* client, int argc, char** argv) {
 
     /* Automatically set HOME environment variable if unset (FreeRDP's
@@ -138,7 +195,7 @@ int guac_client_init(guac_client* client, int argc, char** argv) {
     client->args = GUAC_RDP_CLIENT_ARGS;
 
     /* Alloc client data */
-    guac_rdp_client* rdp_client = calloc(1, sizeof(guac_rdp_client));
+    guac_rdp_client* rdp_client = guac_mem_zalloc(sizeof(guac_rdp_client));
     client->data = rdp_client;
 
     /* Init clipboard */
@@ -164,6 +221,7 @@ int guac_client_init(guac_client* client, int argc, char** argv) {
 
     /* Set handlers */
     client->join_handler = guac_rdp_user_join_handler;
+    client->join_pending_handler = guac_rdp_join_pending_handler;
     client->free_handler = guac_rdp_client_free_handler;
     client->leave_handler = guac_rdp_user_leave_handler;
 
@@ -239,7 +297,7 @@ int guac_rdp_client_free_handler(guac_client* client) {
     pthread_mutex_destroy(&(rdp_client->message_lock));
 
     /* Free client data */
-    free(rdp_client);
+    guac_mem_free(rdp_client);
 
     return 0;
 
