@@ -33,7 +33,35 @@
 
 #include <guacamole/argv.h>
 #include <guacamole/client.h>
+#include <guacamole/mem.h>
 #include <guacamole/recording.h>
+#include <guacamole/socket.h>
+
+/**
+ * A pending join handler implementation that will synchronize the connection
+ * state for all pending users prior to them being promoted to full user.
+ *
+ * @param client
+ *     The client whose pending users are about to be promoted.
+ *
+ * @return
+ *     Always zero.
+ */
+static int guac_telnet_join_pending_handler(guac_client* client) {
+
+    guac_telnet_client* telnet_client = (guac_telnet_client*) client->data;
+
+    /* Synchronize the terminal state to all pending users */
+    if (telnet_client->term != NULL) {
+        guac_socket* broadcast_socket = client->pending_socket;
+        guac_terminal_sync_users(telnet_client->term, client, broadcast_socket);
+        guac_telnet_send_current_argv_batch(client, broadcast_socket);
+        guac_socket_flush(broadcast_socket);
+    }
+
+    return 0;
+
+}
 
 int guac_client_init(guac_client* client) {
 
@@ -41,7 +69,7 @@ int guac_client_init(guac_client* client) {
     client->args = GUAC_TELNET_CLIENT_ARGS;
 
     /* Allocate client instance data */
-    guac_telnet_client* telnet_client = calloc(1, sizeof(guac_telnet_client));
+    guac_telnet_client* telnet_client = guac_mem_zalloc(sizeof(guac_telnet_client));
     client->data = telnet_client;
 
     /* Init telnet client */
@@ -51,6 +79,7 @@ int guac_client_init(guac_client* client) {
 
     /* Set handlers */
     client->join_handler = guac_telnet_user_join_handler;
+    client->join_pending_handler = guac_telnet_join_pending_handler;
     client->free_handler = guac_telnet_client_free_handler;
     client->leave_handler = guac_telnet_user_leave_handler;
 
@@ -97,7 +126,7 @@ int guac_telnet_client_free_handler(guac_client* client) {
     if (telnet_client->settings != NULL)
         guac_telnet_settings_free(telnet_client->settings);
 
-    free(telnet_client);
+    guac_mem_free(telnet_client);
     return 0;
 
 }
