@@ -74,21 +74,6 @@ int guac_vnc_user_join_handler(guac_user* user, int argc, char** argv) {
 
     }
 
-    /* If not owner, synchronize with current state */
-    else {
-
-#ifdef ENABLE_PULSE
-        /* Synchronize an audio stream */
-        if (vnc_client->audio)
-            guac_pa_stream_add_user(vnc_client->audio, user);
-#endif
-
-        /* Synchronize with current display */
-        guac_common_display_dup(vnc_client->display, user, user->socket);
-        guac_socket_flush(user->socket);
-
-    }
-
     /* Only handle events if not read-only */
     if (!settings->read_only) {
 
