@@ -21,8 +21,10 @@
 
 #include <guacamole/client.h>
 #include <guacamole/layer.h>
+#include <guacamole/mem.h>
 #include <guacamole/socket.h>
 #include <guacamole/protocol.h>
+#include <guacamole/user.h>
 
 #include <stdlib.h>
 
@@ -31,7 +33,7 @@ guac_terminal_scrollbar* guac_terminal_scrollbar_alloc(guac_client* client,
 
     /* Allocate scrollbar */
     guac_terminal_scrollbar* scrollbar =
-        malloc(sizeof(guac_terminal_scrollbar));
+        guac_mem_alloc(sizeof(guac_terminal_scrollbar));
 
     /* Associate client */
     scrollbar->client = client;
@@ -81,7 +83,7 @@ void guac_terminal_scrollbar_free(guac_terminal_scrollbar* scrollbar) {
     guac_client_free_layer(scrollbar->client, scrollbar->container);
 
     /* Free scrollbar */
-    free(scrollbar);
+    guac_mem_free(scrollbar);
 
 }
 
@@ -332,7 +334,7 @@ static void calculate_state(guac_terminal_scrollbar* scrollbar,
 }
 
 void guac_terminal_scrollbar_dup(guac_terminal_scrollbar* scrollbar,
-        guac_user* user, guac_socket* socket) {
+        guac_client* client, guac_socket* socket) {
 
     /* Get old state */
     guac_terminal_scrollbar_render_state* state = &scrollbar->render_state;
